import 'dart:async';
import 'dart:io' show Platform;

import 'package:flutter/foundation.dart';
import 'package:in_app_purchase/in_app_purchase.dart';
import 'package:puma_pay/entities/product/product.dart';
import 'package:puma_pay/extensions/extProduct.dart';
import 'package:puma_pay/extensions/extProductDetail.dart';
import 'package:puma_pay/extensions/extPurchaseDetails.dart';
import 'package:puma_pay/helpers/appStore.dart';
import 'package:puma_pay/redux/pumaPayActions.dart';
import 'package:puma_pay/redux/pumaPayState.dart';
import 'package:redux/redux.dart';

initializeGooglePlay() {
  // Inform the plugin that this app supports pending purchases on Android.
  // An error will occur on Android if you access the plugin `instance`
  // without this call.
  if (defaultTargetPlatform == TargetPlatform.android) {
    // InAppPurchaseAndroidPlatformAddition.enablePendingPurchases();
  }
}

isStoreAvailable() async {
  InAppPurchase _iap = InAppPurchase.instance;

  // Check availability of In App Purchases
  final _available = await _iap.isAvailable();
  if (_available) {
    print("🎉 Store Available");
  } else {
    print("🚫 Store not Available");
  }
  return _available;
}

Future<List<Product>> getProducts(List<String> productIDs) async {
  List<Product> products = [];

  print("🛍 Getting products... ${productIDs.toSet()}");
  final ProductDetailsResponse response =
      await InAppPurchase.instance.queryProductDetails(productIDs.toSet());
  print("stop");

  if (response.notFoundIDs.isNotEmpty) {
    print("🚫 Products Not found ${products.length}");
  }
  List<ProductDetails> productDetails = response.productDetails;
  for (var productDetail in productDetails) {
    products.add(productDetail.toProduct());
  }

  print("Products ${products.length}");

  if (products.length > 0) {
    print("Products ${products[0].id} ${products[0].price}");
  }

  return products;
}

purchaseProduct(Product product) {
  print("💶 Purchasing product: ${product.id}");

  final PurchaseParam purchaseParam =
      PurchaseParam(productDetails: product.toProductDetail());

  InAppPurchase.instance.buyConsumable(purchaseParam: purchaseParam).then(
      (value) {
    print("💶 Purchasing Success: $value");
  }, onError: (stacktrace) {
    print("💶 Purchasing Error: $stacktrace");
  });
}

listenToPurchaseUpdated(
    List<PurchaseDetails> purchaseDetailsList, Store store) {
  final VirtualPumaPayState state = store.state;

  final storeName = Platform.isAndroid ? "android" : "ios";

  print(
      "🔔 listenToPurchaseUpdated called with ${purchaseDetailsList.length} items");

  purchaseDetailsList.forEach((PurchaseDetails item) async {
    final _purchaseDetail = item.toPurchaseDetailEntity();

    if (item.status == PurchaseStatus.pending) {
      store.dispatch(SetPurchasePendingAction(purchasePending: true));
    } else {
      store.dispatch(SetPurchasePendingAction(purchasePending: false));
      final receipt = AppStoreReceiptData(
          item.verificationData.localVerificationData,
          state.pumaPayState.appleAppStorePassword!,
          true);

      if (item.status == PurchaseStatus.error) {
        print("🚨 Found item with error status: ${item.toString()}");
        InAppPurchase.instance.completePurchase(item);
      } else if ((item.status == PurchaseStatus.purchased) ||
          item.status == PurchaseStatus.restored) {
        store.dispatch(VerifyInAppPurchaseAction(
            receipt: receipt.toJson(),
            productID: item.productID,
            storeName: storeName,
            purchaseDetail: _purchaseDetail));
      }

      if (item.pendingCompletePurchase) {
        await InAppPurchase.instance.completePurchase(item);
      } else {
        InAppPurchase.instance.completePurchase(item);
      }
    }
  });
}
