library puma_pay_state;

import 'dart:convert';

import 'package:built_collection/built_collection.dart';
import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:puma_pay/entities/product/product.dart';

import '../../serializers.dart';

part 'pumaPayState.g.dart';

abstract class VirtualPumaPayState {
  // ErrorState get errorState;
  PumaPayState get pumaPayState;
}

abstract class PumaPayState
    implements Built<PumaPayState, PumaPayStateBuilder> {
  PumaPayState._();

  factory PumaPayState([updates(PumaPayStateBuilder b)]) = _$PumaPayState;

  static PumaPayState initial() {
    return new PumaPayState((b) => b
          ..purchasePending = false
          ..storeAvailable = false
          ..productsLoading = false
          ..restoringPurchase = false
          ..productIDs = ListBuilder<String>()
          ..products = ListBuilder<Product>()
        // ..receipts = ListBuilder<AppStoreReceiptData>()
        );
  }

  @BuiltValueField(wireName: 'productIDs')
  BuiltList<String> get productIDs;

  @BuiltValueField(wireName: 'products')
  BuiltList<Product> get products;

  @BuiltValueField(wireName: 'purchasePending')
  bool get purchasePending;

  @BuiltValueField(wireName: 'storeAvailable')
  bool get storeAvailable;

  @BuiltValueField(wireName: 'productsLoading')
  bool get productsLoading;

  @BuiltValueField(wireName: 'appleAppStorePassword')
  String? get appleAppStorePassword;

  @BuiltValueField(wireName: 'restoringPurchase')
  bool get restoringPurchase;

  @BuiltValueField(wireName: 'verifyEndpoint')
  String? get verifyEndpoint;

  String toJson() {
    return json
        .encode(serializers.serializeWith(PumaPayState.serializer, this));
  }

  static PumaPayState? fromJson(String jsonString) {
    return serializers.deserializeWith(
        PumaPayState.serializer, json.decode(jsonString));
  }

  static Serializer<PumaPayState> get serializer => _$pumaPayStateSerializer;
}
