import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:intl/intl.dart';
import 'package:puma_pay/entities/purchaseDetail/purchaseDetail.dart';
import 'package:puma_pay/redux/pumaPayState.dart';
import 'package:puma_pay/ui/atoms/appStoreReceiptTile.dart';
import 'package:puma_ui/ui/atoms/fittedTitle.dart';
import 'package:redux/redux.dart';

class PurchaseDetailTemplate<T extends VirtualPumaPayState>
    extends StatelessWidget {
  final PurchaseDetail purchaseDetail;

  PurchaseDetailTemplate({
    required this.purchaseDetail,
  });

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      converter: (Store<T> store) => _ViewModel.create(store),
      builder: (BuildContext context, _ViewModel vm) =>
          _createBody(context, vm),
    );
  }

  Text BlackText(String text) {
    return Text(
      text,
      style: TextStyle(
        color: Colors.black,
      ),
    );
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    var format = new DateFormat("yMd");

    return Padding(
      padding: const EdgeInsets.only(left: 16.0, right: 16.0),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          FittedTile(
            title: "Purchase Detail",
            color: Colors.black,
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              BlackText("ProductID"),
              BlackText(purchaseDetail.productID),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              BlackText("Status"),
              BlackText(purchaseDetail.status),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              BlackText("Valid"),
              BlackText(purchaseDetail.valid.toString()),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              BlackText("PurchaseID"),
              BlackText(purchaseDetail.purchaseID ?? ""),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              BlackText("Transaction Date"),
              purchaseDetail.transactionDate != null
                  ? BlackText(
                      format.format(
                        DateTime.fromMillisecondsSinceEpoch(
                          int.parse(purchaseDetail.transactionDate!),
                        ),
                      ),
                    )
                  : BlackText(""),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              BlackText("Receipt Environment"),
              BlackText(purchaseDetail.receipt?.environment ?? ""),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              BlackText("Receipt Status"),
              BlackText(purchaseDetail.receipt?.status.toString() ?? ""),
            ],
          ),
          Divider(
            color: Colors.black,
          ),
          if (purchaseDetail.receipt != null)
            for (var r in purchaseDetail.receipt!.latestReceiptInfo.toList())
              AppStoreReceiptTile(
                receipt: r,
              ),
        ],
      ),
    );
  }
}

class _ViewModel {
  final PumaPayState pumaPayState;

  _ViewModel(
    this.pumaPayState,
  );

  factory _ViewModel.create(Store<VirtualPumaPayState> store) {
    return _ViewModel(
      store.state.pumaPayState,
    );
  }
}
